package mo.dyna.statusbar

import android.os.Build
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.PluginRegistry.Registrar

class StatusbarPlugin: MethodCallHandler
{
  companion object
  {
    @JvmStatic
    fun registerWith(registrar: Registrar)
    {
      val channel = MethodChannel(registrar.messenger(), "statusbar")
      val plugin = StatusbarPlugin()

      if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP)
      {
        plugin.activity = registrar.activity()
        plugin.default = registrar.activity().window.statusBarColor
      }

      channel.setMethodCallHandler(plugin)
    }
  }

  private var activity: android.app.Activity? = null
  private var color: Int? = null
  private var default: Int? = null
  private var active = false

  override fun onMethodCall(call: MethodCall, result: Result)
  {
    if (call.method == "show")
    {
      if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP)
      {
        if (!active)
        {
          active = true
          val c = color
          if (c != null)
          {
            activity?.window?.statusBarColor = c
          }
        }
      }
    }
    else if(call.method == "hide")
    {
      if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP)
      {
        val c = default
        if (c != null)
        {
          activity?.window?.statusBarColor = c
        }
      }
    }
    else if(call.method == "color")
    {
      if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP)
      {
        color = android.graphics.Color.parseColor(call.argument("hex"))
        if (active)
        {
          val c = color
          if (c != null)
          {
            activity?.window?.statusBarColor = c
          }
        }
      }
    }
    else
    {
      result.notImplemented()
    }
  }
}
